###############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2018 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################
import json
from datetime import datetime

from odoo import api, fields, models


class TrackEvent(models.Model):
    """An event on a tracker, examples include open, complaints.
    """

    _name = "marketing.campaign.tracker.track.event"
    _description = "Tracker Event"
    _log_access = False
    # order is used to compute last_event_type on track
    _order = "event_datetime DESC, type"

    track_id = fields.Many2one(
        comodel_name="marketing.campaign.tracker.track",
        string="Tracker",
        required=True,
        index=True,
    )

    event_datetime = fields.Datetime(
        string="Event Datetime", help="When the event happened", required=True
    )

    # Not a selection to be more flexible
    type = fields.Char(required=True, help="Type of the event")

    # XXX maybe use ir.model.data instead?
    # this is simpler and a script could be done to ex
    xid = fields.Char(help="External identifier")

    json = fields.Text()

    @api.model
    def new(self, event_data):
        """Use to create a new event, the matching is done on the message-id
        (in the message dict)
        :param event_data:
        :type event_data: dict
        """
        message_id = event_data["message"]["headers"]["message-id"]
        # mailgun return the message id without <>
        tracks = self.env["marketing.campaign.tracker.track"].search(
            [
                "|",
                ("message_uuid", "=", message_id),
                ("message_uuid", "=", "<{}>".format(message_id)),
            ]
        )
        for track in tracks:
            self.create(
                {
                    "track_id": track.id,
                    "event_datetime": fields.Datetime.to_string(
                        datetime.utcfromtimestamp(event_data["timestamp"])
                    ),
                    "type": event_data["event"],
                    "xid": event_data["id"],
                    "json": json.dumps(event_data),
                }
            )
