# -*- coding: utf-8 -*-
##############################################################################
#
#    Email marketing click & read tracker
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011, 2015, 2018 XCG Consulting
#                                   (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from odoo import fields, models


class TrackVisit(models.Model):
    """a trackvisit is a small timestamp associated to a trackitem
    Basically each time a client clicks on a tracked URL and gets redirected
    by the tracker a trackvisit will be recorded for the action. This opens-up
    interesting trails of who clicks/views our marketing emails and how many
    times, when, from where...
    """

    _name = "marketing.campaign.tracker.trackvisit"
    _description = "Track Item Visit"
    _order = "visit_time DESC"
    _log_access = False

    _rec_name = "name_clicker"

    # keep a reference to our parent
    trackitem_id = fields.Many2one(
        comodel_name="marketing.campaign.tracker.trackitem",
        string="Track Item",
        ondelete="cascade",
        index=True,
    )
    visit_time = fields.Datetime(required=True, default=fields.Datetime.now)

    track_id = fields.Many2one(
        related="trackitem_id.track_id",
        string="Track",
        readonly=True,
        store=True,
        index=True,
    )

    campaign_name = fields.Char(
        related="trackitem_id.track_id.campaign_id.name",
        string="Marketing Campaign",
        readonly=True,
        store=True,
    )

    campaign_activity_name = fields.Char(
        related="trackitem_id.track_id.campaign_activity_id.name",
        string="Campaign Activity",
        readonly=True,
        store=True,
    )

    real_url = fields.Char(
        related="trackitem_id.real_url",
        string="Real URL",
        readonly=True,
        store=True,
    )

    name_clicker = fields.Char(
        related="trackitem_id.name_clicker",
        string="Resource Name",
        readonly=True,
        store=True,
    )

    email_to = fields.Char(
        related="trackitem_id.track_id.email_to",
        string="Recipient Email",
        readonly=True,
        store=True,
    )

    partner_id = fields.Many2one(
        comodel_name="res.partner",
        related="trackitem_id.track_id.partner_id",
        string="Partner",
        readonly=True,
        store=True,
    )

    partner_name = fields.Char(
        related=("partner_id", "display_name"),
        string="Partner name",
        compute_sudo=True,
        readonly=True,
        store=True,
    )

    sent = fields.Datetime(
        related="trackitem_id.track_id.sent", readonly=True, store=True
    )

    type = fields.Selection(
        related="trackitem_id.type", readonly=True, store=True
    )

    category = fields.Char(
        related="trackitem_id.category", readonly=True, store=True
    )
