# -*- coding: utf-8 -*-
##############################################################################
#
#    Email marketing click & read tracker
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011, 2018 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import HTMLParser
import re
import urllib
import urlparse
from uuid import uuid4

__author__ = "faide"

simple_text_url_re = re.compile(r"(https?://[-a-zA-Z0-9_/.?&;=]+)")
html_href_re = re.compile(r'href="([-a-zA-Z0-9_/.?&=:;%]+)"')
html_imgsrc_qs_re = re.compile(
    r'img( [^ \t\n\r\f\v=]+="[^"]*")* '
    r'src="([-a-zA-Z0-9_/.?&=:]+\?[-a-zA-Z0-9_/.?&=:%]+)"'
)


def __enabled(option):
    """Test that option is not disabled, by comparing it to false, off and
    disabled

    :param option: text to test
    :return: True if the option is considered enabled
    """
    return len(option) > 0 and option[0].lower() not in (
        "false",
        "off",
        "disabled",
    )


def insert_tracker_in_text(
    text, tracker_base, track_id, opt_out_callback=None
):
    """
    ::return:: new text, list of dict for track items
    """
    tracks = []

    def repl(match_obj):
        real_url = match_obj.group(1)
        uid = str(uuid4())
        # TODO remark: the real_url is escaped there
        values = dict(
            trackitem_uuid=uid,
            real_url=real_url,
            track_id=track_id,
            type="link",
        )
        # TODO factorize code
        u = urlparse.urlparse(real_url)
        qs_dict = urlparse.parse_qs(urlparse.urlparse(real_url).query)
        if "opt_out" in qs_dict and __enabled(qs_dict["opt_out"]):
            if opt_out_callback:
                opt_out_callback(values, qs_dict["opt_out"])
            else:
                values["callback"] = "opt_out"
        if "set_opened" in qs_dict and __enabled(qs_dict["set_opened"]):
            values["callback"] = "set_opened"
        if "display" in qs_dict:
            # TODO check that the page exists
            values["display_id"] = int(qs_dict["display"][0])
        if "category" in qs_dict:
            values["category"] = ",".join(qs_dict["category"])
        if "display_id" not in values and (
            "callback" in values or "category" in values
        ):
            for key in ("opt_out", "set_opened", "category"):
                if key in qs_dict:
                    qs_dict.pop(key)
            u = u._replace(query=urllib.urlencode(qs_dict, True))
            values["real_url"] = urlparse.urlunparse(u)
        tracks.append(values)
        return "%s/%s" % (tracker_base, uid)

    return simple_text_url_re.sub(repl, text), tracks


def insert_tracker_in_html(
    html, tracker_base, track_id, opt_out_callback=None
):
    """
    :param insert_query_images: Indicated if the images with query strings
    should be
    """
    tracks = []
    h = HTMLParser.HTMLParser()

    def repl(match_obj):
        real_url = match_obj.group(1)
        uid = str(uuid4())
        values = dict(trackitem_uuid=uid, track_id=track_id, type="link")
        # TODO factorize code
        u = urlparse.urlparse(real_url)
        qs_dict = urlparse.parse_qs(u.query)
        if "opt_out" in qs_dict and __enabled(qs_dict["opt_out"]):
            if opt_out_callback:
                opt_out_callback(values, qs_dict["opt_out"])
            else:
                values["callback"] = "opt_out"
        if "set_opened" in qs_dict and __enabled(qs_dict["set_opened"]):
            values["callback"] = "set_opened"
        if "display" in qs_dict:
            # TODO check that the page exists
            values["display_id"] = int(qs_dict["display"][0])
        if "category" in qs_dict:
            values["category"] = "".join(qs_dict["category"])
        # remove callback keys from real url if
        # not displaying a page instead of redirecting
        if "display_id" not in values and (
            "callback" in values or "category" in values
        ):
            for key in ("opt_out", "set_opened", "category"):
                if key in qs_dict:
                    qs_dict.pop(key)
            u = u._replace(query=urllib.urlencode(qs_dict, True))
            values["real_url"] = urlparse.urlunparse(u)
        else:
            values["real_url"] = h.unescape(real_url)
        tracks.append(values)
        return 'href="%s/%s"' % (tracker_base, uid)

    def repl3(match_obj):
        other_params = match_obj.group(1) or ""
        real_url = match_obj.group(2)
        uid = str(uuid4())
        values = dict(
            trackitem_uuid=uid,
            real_url=real_url,
            track_id=track_id,
            type="image",
        )
        u = urlparse.urlparse(real_url)
        qs_dict = urlparse.parse_qs(u.query)
        if "set_opened" in qs_dict and __enabled(qs_dict["set_opened"]):
            values["callback"] = "set_opened"
        if "category" in qs_dict:
            values["category"] = "".join(qs_dict["category"])
        if "callback" in values or "category" in values:
            for key in ("set_opened", "category"):
                if key in qs_dict:
                    qs_dict.pop(key)
            u = u._replace(query=urllib.urlencode(qs_dict, True))
            values["real_url"] = urlparse.urlunparse(u)
        else:
            values["real_url"] = h.unescape(real_url)
        tracks.append(values)
        return 'img%s src="%s/%s"' % (other_params, tracker_base, uid)

    res = html_href_re.sub(repl, html)
    return html_imgsrc_qs_re.sub(repl3, res), tracks
