# -*- coding: utf-8 -*-
##############################################################################
#
#    Marketing campaign analytic.
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011, 2015 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from odoo import api, fields, models


class campaign_real_url(models.Model):
    _name = "marketing.campaign.tracker.campaign_real_url"

    analytic_id = fields.Many2one(
        comodel_name="marketing.campaign.tracker.analytic"
    )
    real_url = fields.Char(size=255)
    clicks = fields.Integer()
    clicker = fields.Integer()
    timedate = fields.Integer()


class analytic(models.Model):
    """Store and calculate information to display analytic
    graphs of all running marketing campaign
    """

    _name = "marketing.campaign.tracker.analytic"

    @api.model
    def empty_campaign_real_url_table(self):
        campaign_real_url_obj = self.env.get(
            "marketing.campaign.tracker.campaign_real_url"
        )
        campaign_real_urls = campaign_real_url_obj.search([])
        campaign_real_urls.unlink()

    @api.model
    def count_real_url(self, campaign_id):
        trackitem_obj = self.env.get("marketing.campaign.tracker.trackitem")
        trackitems = trackitem_obj.search([("campaign_id", "=", campaign_id)])

        campaign_real_url_obj = self.env.get(
            "marketing.campaign.tracker.campaign.real_url"
        )
        self.empty_campaign_real_url_table()
        analytic_id = self.search([("campaign_id", "=", campaign_id)])
        for trackitem in trackitems:
            url_count = 0
            is_in_table = campaign_real_url_obj.search(
                [("real_url", "=", trackitem.real_url)]
            )
            if not is_in_table:
                url_count = trackitem_obj.search(
                    [("real_url", "=", trackitem.real_url)], count=True
                )
                campaign_real_url_obj.create(
                    {
                        "analytic_id": analytic_id[0],
                        "real_url": trackitem.real_url,
                        "clicks": url_count,
                    }
                )

    @api.model
    def update_graphs(self, campaign_id):
        analytic_id = self.search([("campaign_id", "=", campaign_id)])
        self.count_real_url(campaign_id)
        if not analytic_id:
            self.create({"campaign_id": campaign_id})
        return 0

    campaign_id = fields.Integer(string="Campaign ID")

    urls = fields.One2many(
        comodel_name="marketing.campaign.tracker.campaign_real_url",
        inverse_name="analytic_id",
        string="URLs",
    )
