# -*- coding: utf-8 -*-
###############################################################################
#
#    Odoo, Open Source Management Solution
#    Copyright (C) 2015 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import odoo


class TestMarketingCampaign(odoo.tests.common.TransactionCase):
    """Unit test similar to those done by OCA
    """

    def test_insert_trackers(self):

        track_obj = self.env["marketing.campaign.tracker.track"]

        url = (
            "http://some.org/a_3/super-script.php?uid=3&someval=4&opt_out=True"
            "&display=1"
        )
        text = url
        base_html = '<a href="%s"> Some Super Promo </a>'
        html = (
            '<a href="http://some.org/a_3/super-script.php?uid=3&amp;someval=4'
            '&amp;opt_out=True&amp;display=1"> Some Super Promo </a>'
        )

        new_text, values = track_obj.insert_tracker_in_text(text, "BASE", "1")

        self.assertEqual(
            len(values), 1, "Values should contain only one record"
        )

        value = values[0]
        track_uuid = value.get("trackitem_uuid", None)

        self.assertNotEqual(track_uuid, None, "Track uuid should not be None")

        real_url = value.get("real_url", None)

        self.assertEqual(real_url, url)
        self.assertEqual(new_text, "%s/%s" % ("BASE", track_uuid))

        new_html, values = track_obj.insert_tracker_in_html(html, "BASE", "1")

        self.assertEqual(
            len(values),
            1,
            "Values should have one record, not %s" % len(values),
        )

        value = values[0]
        track_uuid = value.get("trackitem_uuid", None)

        self.assertNotEqual(track_uuid, None, "Track uuid should not be None")

        real_url = value.get("real_url", None)

        self.assertEqual(real_url, url)
        expected_html = base_html % ("BASE/" + track_uuid)
        self.assertEqual(new_html, expected_html)

        value = values[0]
        track_uuid = value.get("trackitem_uuid", None)
        self.assertNotEqual(track_uuid, None, "Track uuid should not be None")
        self.assertEqual(value["callback"], "opt_out")
        self.assertEqual(value["display_id"], 1)

        url = "http://some.org/a_3/super-script.php?uid=3&someval=4"
        url2 = "http://some.org/a_3/super-script.php?someval=4&uid=3"
        base_html = '<a href="%s"> Some Super Promo </a>'
        html = (
            '<a href="http://some.org/a_3/super-script.php?uid=3&amp;someval=4'
            '&amp;opt_out=True"> Some Super Promo </a>'
        )

        new_html, values = track_obj.insert_tracker_in_html(html, "BASE", "1")

        self.assertEqual(
            len(values),
            1,
            "Values should have one record, not %s" % len(values),
        )

        value = values[0]
        track_uuid = value.get("trackitem_uuid", None)

        self.assertNotEqual(track_uuid, None, "Track uuid should not be None")

        real_url = value.get("real_url", None)

        self.assertIn(real_url, [url, url2])
        expected_html = base_html % ("BASE/" + track_uuid)
        self.assertEqual(new_html, expected_html)

        value = values[0]
        track_uuid = value.get("trackitem_uuid", None)
        self.assertNotEqual(track_uuid, None, "Track uuid should not be None")
        self.assertEqual(value["callback"], "opt_out")
        self.assertNotIn("display_id", value)

    def test_insert_image_no_qs(self):
        """Test image tracker insertion, case where there is no query string
        """
        track_obj = self.env["marketing.campaign.tracker.track"]
        img_url = "https://website.paris/b_5/super-img.png"
        base_img = '<img src="%s" />'
        imgsrc = base_img % img_url

        new_html, values = track_obj.insert_tracker_in_html(
            imgsrc, "BASE", "1"
        )

        self.assertEqual(len(values), 0, "Values should have no record")
        self.assertEqual(new_html, imgsrc)

    def test_insert_images1(self):
        """Test image tracker insertion
        """
        img_url = "http://some.org/b_5/super-img.php?img=45&value=33"
        base_img = '<img src="%s" />'
        imgsrc = base_img % img_url
        self.insert_images_call_and_assert(img_url, base_img, imgsrc)

    def test_insert_images2(self):
        """Test image tracker insertion
        """
        img_url = "http://some.org/b_5/super-img.php?img=45&value=33"
        base_img = '<img alt="Test" src="%s" />'
        imgsrc = base_img % img_url
        self.insert_images_call_and_assert(img_url, base_img, imgsrc)

    def test_insert_images3(self):
        """Test image tracker insertion, with removal of callback
        """
        img_url = "http://some.org/b_5/super-img.php?img=45&set_opened=True"
        img_url_real = "http://some.org/b_5/super-img.php?img=45"
        base_img = '<img src="%s" />'
        imgsrc = base_img % img_url
        self.insert_images_call_and_assert(img_url_real, base_img, imgsrc)

    def test_insert_images_doc(self):
        """Test image tracker insertion, as mentioned in read me
        """
        img_real_url = "http://example.org/b_5/my_image.png"
        img_url = img_real_url + "?set_opened=True"
        base_img = '<img src="%s" />'
        imgsrc = base_img % img_url
        self.insert_images_call_and_assert(img_real_url, base_img, imgsrc)

    def insert_images_call_and_assert(self, url, basehtml, html):
        track_obj = self.env["marketing.campaign.tracker.track"]
        new_html, values = track_obj.insert_tracker_in_html(html, "BASE", "1")

        self.assertEqual(len(values), 1, "Values should have one record")

        value = values[0]
        track_uuid = value.get("trackitem_uuid", None)

        self.assertNotEqual(track_uuid, None, "Track uuid should not be None")

        real_url = value.get("real_url", None)

        self.assertEqual(real_url, url)
        expected_html = basehtml % ("BASE/" + track_uuid)
        self.assertEqual(new_html, expected_html)
        return value

    def test_link_opened_removed(self):
        """Test that in a link with set_opened, the value is removed from real
        URL
        """
        url = "http://example.org/b%205/"
        basetext = '<a href="%s">A link</a>'
        src = basetext % (url + "?set_opened=True")
        self.insert_images_call_and_assert(url, basetext, src)

    def test_link_opened_and_other_qs_removed(self):
        """Test that in a link with set_opened and other arguments, the value
        is removed from real URL
        """
        url = "http://example.org/b%205/?stuff=a&b=3"
        basetext = '<a href="%s">A link</a>'
        src = basetext % (
            "http://example.org/b%205/?stuff=a&amp;b=3&amp;set_opened=True"
        )
        self.insert_images_call_and_assert(url, basetext, src)

    def test_link_category(self):
        url = "http://example.org/b%205/?stuff=a&b=3"
        basetext = '<a href="%s">A link</a>'
        src = basetext % (
            "http://example.org/b%205/?stuff=a&amp;b=3&amp;category=Something"
        )
        value = self.insert_images_call_and_assert(url, basetext, src)
        self.assertEqual("Something", value["category"])

    def test_link_category_accent(self):
        url = "http://example.org/b%205/"
        basetext = '<a href="%s">A link</a>'
        src = basetext % ("http://example.org/b%205/?category=%C3%89")
        value = self.insert_images_call_and_assert(url, basetext, src)
        self.assertEqual("É", value["category"])
