###############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2019 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################
from odoo import fields
from odoo.tests import TransactionCase


class TestTrack(TransactionCase):
    def test_create_batch_minimal(self):
        track_model = self.env["marketing.campaign.tracker.track"]
        vals = {
            "campaign_activity_id": self.ref(
                "marketing_campaign.marketing_campaign_activity_0"
            )
        }
        track_id = track_model._create_batch(vals)
        track = track_model.browse(track_id)
        for key, value in vals.items():
            self.assertEqual(getattr(track, key), value)
        self.assertEqual(
            track.campaign_id.id,
            self.ref(
                "marketing_campaign.marketing_campaign_openerppartnerchannel"
            ),
        )
        self.assertTrue(track.create_uid)
        self.assertTrue(track.write_uid)
        self.assertTrue(track.create_date)
        self.assertTrue(track.write_date)

    def test_create_batch_simple(self):
        track_model = self.env["marketing.campaign.tracker.track"]
        vals = {
            "campaign_activity_id": self.ref(
                "marketing_campaign.marketing_campaign_activity_1"
            ),
            "template_model_name": "foo",
            "email_to": "foobar@example.com",
            "partner_id": self.ref("base.partner_demo"),
            "res_id": 42,
            "message_uuid": "test",
            "email_template_id": self.ref(
                "marketing_campaign.email_template_1"
            ),
        }
        track_id = track_model._create_batch(vals)
        track = track_model.browse(track_id)
        for key, value in vals.items():
            self.assertEqual(getattr(track, key), value)
        self.assertEqual(
            track.campaign_id.id,
            self.ref(
                "marketing_campaign.marketing_campaign_openerppartnerchannel"
            ),
        )
        self.assertEqual(track.last_event_type, "")

    def test_create_batch_opened(self):
        track_model = self.env["marketing.campaign.tracker.track"]
        vals = {
            "campaign_activity_id": self.ref(
                "marketing_campaign.marketing_campaign_activity_1"
            ),
            "template_model_name": "foo",
            "email_to": "foobar@example.com",
            "partner_id": self.ref("base.partner_demo"),
            "res_id": 42,
            "message_uuid": "test",
            "email_template_id": self.ref(
                "marketing_campaign.email_template_1"
            ),
            "opened": fields.Datetime.now(),
        }
        track_id = track_model._create_batch(vals)
        track = track_model.browse(track_id)
        for key, value in vals.items():
            self.assertEqual(getattr(track, key), value)
        self.assertEqual(
            track.campaign_id.id,
            self.ref(
                "marketing_campaign.marketing_campaign_openerppartnerchannel"
            ),
        )
        self.assertEqual(track.last_event_type, "open")
